#!/bin/sh
#
#-------------------------------------------------------------------
# PROPRIETARY-STATEMENT:
# Licensed Material - Property of IBM
#
# 5724-I63, 5724-H88, 5655-N01, 5733-W61
# (C) Copyright IBM Corp. 1999, 2011
# All Rights Reserved
# US Government Users Restricted Rights - Use, duplication or
# disclosure restricted by GSA ADP Schedule Contract with IBM Corp.
#                                                                               
# Change Activity                                              
#   37754      V1.4.3  20110201  jmierze: Initial version
#-------------------------------------------------------------------------------
#
# This script mounts a file system for use by IBM Installation Manager
#
# The following input arguments are supported:
# 
#   -name
#        Required value that specifies the name of the file system to be 
#        mounted.  This value will be converted to uppercase.
#   -type 
#        Optional value (either "HFS" or "ZFS") that indicates the type of file
#        system to mount.  If not specified, "HFS" is assumed.
#   -mountpoint
#        Required value that specifies the path name of the directory to which 
#        the file system is to be mounted. The directory will be created if this		 
#        value is specified and the directory does not already exist.  (Any
#        missing "parent" directories will also be created.) 
#   -owner
#        Optional value that specifies the user ID of the owner of the file	   
#        system.  If specified, the chown command will be used to set file
#        ownership for all files in the file system.
#   -group
#        Optional value that specifies the group ID of the owner of the file 
#        system.    If specified, the chown command will be used to set file
#        ownership for all files in the file system.
#   -perm
#        Optional value that specifies the file permissions for the files in the
#        file system.  If specified, the chmod command will be used to set file 
#        permissions for all files in the filesystem.
#
#-------------------------------------------------------------------------------

# Prints usage statement
function usage {
	echo "Usage: `basename $0` -name <fileSystemName>" 
	echo "       -mountpoint <mountPoint> [-type <HFS|ZFS>]"
      echo "       [-owner <owner>] [-group <group>] [-perm <permissions>]"
	echo " "
}

# Prints specified message
function printMessage {
	case "$1" in
		"CWLCS9000E" ) echo "$1 $2 value was not specified." ;;
		"CWLCS9001E" ) echo "$1 Error performing TSO allocate for data set $2 on $3." ;;
		"CWLCS9002E" ) echo "$1 Error defining VSAM linear data set $2 on $3." ;;
		"CWLCS9003E" ) echo "$1 Error formatting VSAM linear data set $2." ;;
		"CWLCS9004I" ) echo "$1 Allocating file system data set $2." ;;
		"CWLCS9005I" ) echo "$1 File system data set $2 successfully allocated." ;;
		"CWLCS9006I" ) echo "$1 Mounting data set $2 at mount point $3." ;;
		"CWLCS9007I" ) echo "$1 $2 successfully mounted at mount point $3." ;;
		"CWLCS9008E" ) echo "$1 Mount for $2 at mount point $3 failed." ;;
		"CWLCS9009W" ) echo "$1 Unrecognized argument $2." ;;
		"CWLCS9010I" ) echo "$1 Mount point $2 already exists and is a directory." ;;
		"CWLCS9011E" ) echo "$1 mountPoint path name already exists but is not a directory." ;;
		"CWLCS9012I" ) echo "$1 Creating mount point directory $2." ;;
		"CWLCS9013I" ) echo "$1 Mount point directory $2 successfully created." ;;
		"CWLCS9014E" ) echo "$1 Unable to create mount point directory $2." ;;
		"CWLCS9015E" ) echo "$1 $2 value does not begin with /." ;;
		"CWLCS9016E" ) echo "$1 Unable to set owner and group for directory $2." ;;
		"CWLCS9017I" ) echo "$1 Setting owner and group for directory $2." ;;
		"CWLCS9018I" ) echo "$1 Owner and group successfully set for directory $2." ;;
		"CWLCS9019I" ) echo "$1 Setting permissions for directory $2." ;;
		"CWLCS9020I" ) echo "$1 Permissions succesfully set for directory $2." ;;
		"CWLCS9021E" ) echo "$1 Unable to set permissions for directory $2." ;;
		"CWLCS9022I" ) echo "$1 Formatting ZFS file system $2." ;;
		"CWLCS9023I" ) echo "$1 Defining file system $2." ;;
		"CWLCS9024I" ) echo "$1 File system $2 successfully defined." ;;
		"CWLCS9025E" ) echo "$1 fileSystemType must be set to either \"HFS\" or \"ZFS\"." ;;
		"CWLCS9026E" ) echo "$1 You must specify either -cylinders or -megabytes." ;;
		"CWLCS9027E" ) echo "$1 Do not specify both -cylinders and -megabytes." ;;
	*) exit 8
	esac
}

# Set defaults 
fileSystemType="HFS"

# Fail with a usage statement if no arguments specified
if [[ -z "$1" ]]; then
    usage
    exit 8
fi

# Parse command line arguments
while [[ ! -z "$1" ]]; do
    case "$1" in
        -name) 
			fileSystemName="$2"
			if [[ -z "$fileSystemName" ]]; then
				printMessage CWLCS9000E FileSystemName
				exit 8
			fi
			firstChar=`expr substr "$fileSystemName" 1 1`
			if [[ "$firstChar" == "-" ]]; then
				printMessage CWLCS9000E FileSystemName
				usage
				exit 8
			fi
			shift
            ;;
        -type) 
			fileSystemType="$2"
			if [[ -z "$fileSystemType" ]]; then
				printMessage CWLCS9000E FileSystemType
				exit 8
			fi
			firstChar=`expr substr "$fileSystemType" 1 1`
			if [[ "$firstChar" == "-" ]]; then
				printMessage CWLCS9000E FileSystemType
				exit 8
			fi
			shift
            ;;
        -mountpoint) 
			mountPoint="$2"
			if [[ -z "$mountPoint" ]]; then
				printMessage CWLCS9000E mountPoint
				exit 8
			fi
			firstChar=`expr substr "$mountPoint" 1 1`
			if [[ "$firstChar" == "-" ]]; then
				printMessage CWLCS9000E mountPoint
				exit 8
			fi
			shift
            ;;
        -owner) 
			owner="$2"
			if [[ -z "$owner" ]]; then
				printMessage CWLCS9000E owner
				exit 8
			fi
			firstChar=`expr substr "$owner" 1 1`
			if [[ "$firstChar" == "-" ]]; then
				printMessage CWLCS9000E owner
				exit 8
			fi
			shift
            ;;
        -group) 
			group="$2"
			if [[ -z "$group" ]]; then
				printMessage CWLCS9000E group
				exit 8
			fi
			firstChar=`expr substr "$group" 1 1`
			if [[ "$firstChar" == "-" ]]; then
				printMessage CWLCS9000E group
				exit 8
			fi
			shift
            ;;
        -perm) 
			permissions="$2"
			if [[ -z "$permissions" ]]; then
				printMessage CWLCS9000E permissions
				exit 8
			fi
			firstChar=`expr substr "$permissions" 1 1`
			if [[ "$firstChar" == "-" ]]; then
				printMessage CWLCS9000E permissions
				exit 8
			fi
			shift
            ;;
		-h | -help)	
			usage
			exit
			;;
        *)             
			printMessage CWLCS9009W "$1"
                  exit
    esac
    shift
done

# Verifies fileSystemName specified
if [[ -z "$fileSystemName" ]]; then
    printMessage CWLCS9000E fileSystemName
    exit 8
fi

# Convert the file system name to upper case
fileSystemName=`echo $fileSystemName | tr [a-z] [A-Z]`

# Verifies mountPoint specified
if [[ -z "$mountPoint" ]]; then
    printMessage CWLCS9000E mountPoint
    exit 8
fi

# Verifies fileSystemType 
if [[ "$fileSystemType" != "ZFS" && "$fileSystemType" != "HFS" ]]; then
	printMessage CWLCS9025E
	exit 8
fi

# Verification that is dependent on the mount point being specified
if [[ ! -z "$mountPoint" ]]; then

    # Verifies absolute path specified for mount point
    slashIndex=`expr index "$mountPoint" "\/"`
    if [[ "$slashIndex" -ne 1 ]]; then
        printMessage CWLCS9015E mountPoint
        exit 8
    fi

    # Mount point path name exists but is not a directory
    if [[ -e "$mountPoint" && ! -d  "$mountPoint" ]]; then
		printMessage CWLCS9011E 
        exit 8
    fi
fi

# Computes path name of directory at which ownership and permissions will 
# be set. This will be the highest directory in the path name that is
# created (or the mount point directory if the mount point directory
# already exists).
ownerPoint="$mountPoint"
currentDir="$mountPoint"
while [[ ! -d "$currentDir" ]]; do
	ownerPoint="$currentDir"
	currentDir=`dirname "$ownerPoint"`
done

# Mount point directory exists
if [[ -d "$mountPoint" ]]; then
     printMessage CWLCS9010I "$mountPoint"
	
# Mount point directory does not exist - create mount point
   else
        printMessage CWLCS9012I "$mountPoint"
        mkdir -p -m 775 "$mountPoint"
        rc=$?
	    if [ "$rc" -eq 0 ]; then
            printMessage CWLCS9013I "$mountPoint"
        else
            printMessage CWLCS9014E "$mountPoint"
            exit "$rc"
        fi
 fi 

# Mount configuration file system
printMessage "CWLCS9006I" "$fileSystemName" "$mountPoint"
if [[ "$fileSystemType" == "HFS" ]]; then
	/usr/sbin/mount -f "$fileSystemName" -t HFS -a no "$mountPoint"
	rc=$?
else
	/usr/sbin/mount -f "$fileSystemName" -t ZFS -a no -o 'AGGRGROW' "$mountPoint"
	rc=$?
fi

if [ "$rc" -ne 0 ]; then
	printMessage CWLCS9008E "$fileSystemName" "$mountPoint"
	exit "$rc"
fi
printMessage CWLCS9007I "$fileSystemName" "$mountPoint"
	
# If mount point is a symbolic link point to the actual directory
if [[ -L "$ownerPoint" ]]; then
	curDir=`pwd`
	cd "$ownerPoint"
	ownerPoint=`pwd`
	cd "$curDir"
fi
	
# Sets the ownership on all directories that were created (if no directories 
# were created then only the mount point directory will be updated)
if [[ ! -z "$owner$group" ]]; then
	printMessage CWLCS9017I "$ownerPoint"
	chown -Rh "$owner":"$group" "$ownerPoint"
	rc=$?
	if [ "$rc" -eq 0 ]; then
		printMessage CWLCS9018I "$ownerPoint"
	else
		printMessage CWLCS9016E "$ownerPoint"
		exit "$rc"
	fi
fi
	
# Sets the permissions on all directories that were created (if no  
# directories were created then only the mount point directory will be 
# updated)
if [[ ! -z "$permissions" ]]; then
    printMessage CWLCS9019I "$ownerPoint"
      chmod -R "$permissions" "$ownerPoint"
	rc=$?
	if [ "$rc" -eq 0 ]; then
		printMessage CWLCS9020I "$ownerPoint"
	else
		printMessage CWLCS9021E "$ownerPoint"
		exit "$rc"
	fi
fi
		
exit 0	